#!/usr/bin/env python
# -*- coding: utf-8 -*-

# GIMP plugin to create a new line art image framed (using G'MIC filters)

# (c) Dinasset 2023
#
#   History:
#
#   r0v0.0: 2023-06: Initial version
#   r0v0.1: 2023-06: added user choice of frame color  and size (by MareroQ)
#         		  + some improvements
#   r0v1.0: 2023-07: added gegl support (used for gegl-median-blur)
#   r0v2.S: 2023-07: maintained only styles which give good 
#                    improved general process and single styles
#                    added the possibiity to select the intensity of the final outcome
#   ver.2.S_P: 2023-07:  add Preview a few changes required for the Preview to work for more control over the result (by MareroQ)

#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
#####################################################################################################################################################
############################################# START fu_preview 0.1.1 by Claude Lion $ J.F. Garcia ###################################################
#####################################################################################################################################################

import string as _string
import math
import gimp
import gimpcolor
import colorsys
import re
from gimpenums import *
pdb = gimp.pdb
import os, time, sys
import gettext
t = gettext.translation("gimp20-python", gimp.locale_directory, fallback=True)	#MrQ for translate in dir "locle"
_ = t.ugettext																	#MrQ for translate in dir "locle"
import random
class error(RuntimeError): pass
class CancelError(RuntimeError): pass

PF_INT8        = PDB_INT8
PF_INT16       = PDB_INT16
PF_INT32       = PDB_INT32
PF_INT         = PF_INT32
PF_FLOAT       = PDB_FLOAT
PF_STRING      = PDB_STRING
PF_VALUE       = PF_STRING
PF_COLOR       = PDB_COLOR
PF_COLOUR      = PF_COLOR
PF_ITEM        = PDB_ITEM
PF_DISPLAY     = PDB_DISPLAY
PF_IMAGE       = PDB_IMAGE
PF_LAYER       = PDB_LAYER
PF_CHANNEL     = PDB_CHANNEL
PF_DRAWABLE    = PDB_DRAWABLE
PF_VECTORS     = PDB_VECTORS

PF_TOGGLE      = 1000
PF_BOOL        = PF_TOGGLE
PF_SLIDER      = 1001
PF_SPINNER     = 1002
PF_ADJUSTMENT  = PF_SPINNER

PF_FONT        = 1003
PF_FILE        = 1004
PF_BRUSH       = 1005
PF_PATTERN     = 1006
PF_GRADIENT    = 1007
PF_RADIO       = 1008
PF_TEXT        = 1009
PF_PALETTE     = 1010
PF_FILENAME    = 1011
PF_DIRNAME     = 1012
PF_OPTION      = 1013
PF_TAB         = 1014
PF_DRAW        = 1015
PF_TITLE       = 1016

_type_mapping = {
    PF_INT8        : PDB_INT8,
    PF_INT16       : PDB_INT16,
    PF_INT32       : PDB_INT32,
    PF_FLOAT       : PDB_FLOAT,
    PF_STRING      : PDB_STRING,
    PF_COLOR       : PDB_COLOR,
    PF_ITEM        : PDB_ITEM,
    PF_DISPLAY     : PDB_DISPLAY,
    PF_IMAGE       : PDB_IMAGE,
    PF_LAYER       : PDB_LAYER,
    PF_CHANNEL     : PDB_CHANNEL,
    PF_DRAWABLE    : PDB_DRAWABLE,
    PF_VECTORS     : PDB_VECTORS,

    PF_TOGGLE      : PDB_INT32,
    PF_SLIDER      : PDB_FLOAT,
    PF_SPINNER     : PDB_INT32,

    PF_FONT        : PDB_STRING,
    PF_FILE        : PDB_STRING,
    PF_BRUSH       : PDB_STRING,
    PF_PATTERN     : PDB_STRING,
    PF_GRADIENT    : PDB_STRING,
    PF_RADIO       : PDB_STRING,
    PF_TEXT        : PDB_STRING,
    PF_PALETTE     : PDB_STRING,
    PF_FILENAME    : PDB_STRING,
    PF_DIRNAME     : PDB_STRING,
    PF_OPTION      : PDB_INT32,
    PF_TAB         : PDB_STRING,
    PF_DRAW        : PDB_INT32,
    PF_TITLE       : PDB_STRING,
}

_obj_mapping = {
    PF_INT8        : int,
    PF_INT16       : int,
    PF_INT32       : int,
    PF_FLOAT       : float,
    PF_STRING      : str,
    PF_COLOR       : gimpcolor.RGB,
    PF_ITEM        : int,
    PF_DISPLAY     : gimp.Display,
    PF_IMAGE       : gimp.Image,
    PF_LAYER       : gimp.Layer,
    PF_CHANNEL     : gimp.Channel,
    PF_DRAWABLE    : gimp.Drawable,
    PF_VECTORS     : gimp.Vectors,

    PF_TOGGLE      : bool,
    PF_SLIDER      : float,
    PF_SPINNER     : int,

    PF_FONT        : str,
    PF_FILE        : str,
    PF_BRUSH       : str,
    PF_PATTERN     : str,
    PF_GRADIENT    : str,
    PF_RADIO       : str,
    PF_TEXT        : str,
    PF_PALETTE     : str,
    PF_FILENAME    : str,
    PF_DIRNAME     : str,
    PF_OPTION      : int,
    PF_TAB         : str,
    PF_DRAW        : int,
    PF_TITLE       : str,
}

_registered_plugins_ = {}


def register(proc_name, blurb, help, author, copyright, date, label,
             imagetypes, params, results, function,
             menu=None, domain=None, on_query=None, on_run=None):
    """This is called to register a new plug-in."""

    # First perform some sanity checks on the data
    def letterCheck(str):
        allowed = _string.letters + _string.digits + "_" + "-"
        for ch in str:
            if not ch in allowed:
                return 0
        else:
            return 1

    if not letterCheck(proc_name):
        raise error, "procedure name contains illegal characters"

    for ent in params:
        if len(ent) < 4:
            raise error, ("parameter definition must contain at least 4 "
                          "elements (%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != int:
            raise error, "parameter types must be integers"

        if not letterCheck(ent[1]):
            raise error, "parameter name contains illegal characters"

    for ent in results:
        if len(ent) < 3:
            raise error, ("result definition must contain at least 3 elements "
                          "(%s given: %s)" % (len(ent), ent))

        if type(ent[0]) != type(42):
            raise error, "result types must be integers"

        if not letterCheck(ent[1]):
            raise error, "result name contains illegal characters"

    plugin_type = PLUGIN

    if (not proc_name.startswith("python-") and
        not proc_name.startswith("python_") and
        not proc_name.startswith("extension-") and
        not proc_name.startswith("extension_") and
        not proc_name.startswith("plug-in-") and
        not proc_name.startswith("plug_in_") and
        not proc_name.startswith("file-") and
        not proc_name.startswith("file_")):
           proc_name = "python-fu-" + proc_name

    # if menu is not given, derive it from label
    need_compat_params = False
    if menu is None and label:
        fields = label.split("/")
        if fields:
            label = fields.pop()
            menu = "/".join(fields)
            need_compat_params = True

            import warnings
            message = ("%s: passing the full menu path for the menu label is "
                       "deprecated, use the 'menu' parameter instead"
                       % (proc_name))
            warnings.warn(message, DeprecationWarning, 3)

        if need_compat_params and plugin_type == PLUGIN:
            file_params = [(PDB_STRING, "filename", "The name of the file", ""),
                           (PDB_STRING, "raw-filename", "The name of the file", "")]

            if menu is None:
                pass
            elif menu.startswith("<Load>"):
                params[0:0] = file_params
            elif menu.startswith("<Image>") or menu.startswith("<Save>"):
                params.insert(0, (PDB_IMAGE, "image", "Input image", None))
                params.insert(1, (PDB_DRAWABLE, "drawable", "Input drawable", None))
                if menu.startswith("<Save>"):
                    params[2:2] = file_params

    _registered_plugins_[proc_name] = (blurb, help, author, copyright,
                                       date, label, imagetypes,
                                       plugin_type, params, results,
                                       function, menu, domain,
                                       on_query, on_run)

def _query():
    for plugin in _registered_plugins_.keys():
        (blurb, help, author, copyright, date,
         label, imagetypes, plugin_type,
         params, results, function, menu, domain,
         on_query, on_run) = _registered_plugins_[plugin]

        def make_params(params):
            return [(_type_mapping[x[0]],
                     x[1],
                     _string.replace(x[2], "_", "")) for x in params]

        params = make_params(params)
        # add the run mode argument ...
        params.insert(0, (PDB_INT32, "run-mode",
                                     "The run mode { RUN-INTERACTIVE (0), RUN-NONINTERACTIVE (1) }"))

        results = make_params(results)

        if domain:
            try:
                (domain, locale_dir) = domain
                gimp.domain_register(domain, locale_dir)
            except ValueError:
                gimp.domain_register(domain)

        gimp.install_procedure(plugin, blurb, help, author, copyright,
                               date, label, imagetypes, plugin_type,
                               params, results)

        if menu:
            gimp.menu_register(plugin, menu)
        if on_query:
            on_query()

def getXtra(xtra, valName, defValue):
    p = defValue
    if xtra.get(valName):
        p = xtra.get(valName)
    return p

def _get_defaults(proc_name):
    import gimpshelf

    (blurb, help, author, copyright, date,
     label, imagetypes, plugin_type,
     params, results, function, menu, domain,
     on_query, on_run) = _registered_plugins_[proc_name]

    key = "python-fu-save--" + proc_name

    if gimpshelf.shelf.has_key(key):
        return gimpshelf.shelf[key]
    else:
        # return the default values
        return [x[3] for x in params]

def _set_defaults(proc_name, defaults):
    import gimpshelf

    key = "python-fu-save--" + proc_name
    gimpshelf.shelf[key] = defaults

def _interact(proc_name, start_params):
    (blurb, help, author, copyright, date, label, imagetypes, plugin_type, params, results, function, menu, domain, on_query, on_run) = _registered_plugins_[proc_name]

    def run_script(run_params):
        params = start_params + tuple(run_params)
        _set_defaults(proc_name, params)
        return apply(function, params)

    params = params[len(start_params):]

    # short circuit for no parameters ...
    if len(params) == 0:
         return run_script([])

    import pygtk
    pygtk.require('2.0')

    import gimpui, gtk
#    import pango
    gimpui.gimp_ui_init () # add MrQ
    defaults = _get_defaults(proc_name)
    defaults = defaults[len(start_params):]

    class EntryValueError(Exception):
        pass

    def warning_dialog(parent, primary, secondary=None):
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_CLOSE, primary)
        if secondary:
            dlg.format_secondary_text(secondary)
        dlg.run()
        dlg.destroy()

    def error_dialog(parent, proc_name):
        import sys, traceback

        exc_str = exc_only_str = _("Missing exception information")
        try:
            etype, value, tb = sys.exc_info()
            exc_str = "".join(traceback.format_exception(etype, value, tb))
            exc_only_str = "".join(traceback.format_exception_only(etype, value))
        finally:
            etype = value = tb = None

        title = _("An error occurred running %s") % proc_name
        dlg = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, title)
        dlg.format_secondary_text(exc_only_str)

        alignment = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
        alignment.set_padding(0, 0, 12, 12)
        dlg.vbox.pack_start(alignment)
        alignment.show()

        expander = gtk.Expander(_("_More Information"));
        expander.set_use_underline(True)
        expander.set_spacing(6)
        alignment.add(expander)
        expander.show()

        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.set_size_request(-1, 200)
        expander.add(scrolled)
        scrolled.show()

        label = gtk.Label(exc_str)
        label.set_alignment(0.0, 0.0)
        label.set_padding(6, 6)
        label.set_selectable(True)
        scrolled.add_with_viewport(label)
        label.show()

        def response(widget, id):
            widget.destroy()

        dlg.connect("response", response)
        dlg.set_resizable(True)
        dlg.show()

    # define a mapping of param types to edit objects ...
    class StringEntry(gtk.Entry):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.Entry.__init__(self)
            self.set_text(str(default))
            self.set_activates_default(True)
            #~ self.connect('paste-clipboard', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('inserted-text', lambda _1, _2, _3, _4: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            self.get_buffer().connect('deleted-text', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate()) # modif C.L

        def get_value(self):
            return self.get_text()


    class LabelTitle(gtk.Label):
        #def __init__ (self, default="", **kwargs): # modif C.L. ???
        def __init__(self, default="", align = (0.0, 0.0)):		
            gtk.Label.__init__(self)
            self.set_label(str(default))
            self.set_use_markup(True)
            if align != None:
                self.set_alignment(align[0],align[1])
        def get_value(self):
            return self.get_label()

    class TextEntry(gtk.ScrolledWindow):
        def __init__ (self, default="", **kwargs): # modif C.L.
            gtk.ScrolledWindow.__init__(self)
            self.set_shadow_type(gtk.SHADOW_IN)

            self.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
            self.set_size_request(100, -1)

            self.view = gtk.TextView()
            self.add(self.view)
            self.view.show()
            self.buffer = self.view.get_buffer()
            self.set_value(str(default))
            self.buffer.connect('changed', lambda _: kwargs['dialg'].la_previ.invalidate()) # modif C.L.			

        def set_value(self, text):
            self.buffer.set_text(text)

        def get_value(self):
            return self.buffer.get_text(self.buffer.get_start_iter(), self.buffer.get_end_iter())

    class IntEntry(StringEntry):
        def get_value(self):
            try:
                return int(self.get_text())
            except ValueError, e:
                raise EntryValueError, e.args

    class FloatEntry(StringEntry):
            def get_value(self):
                try:
                    return float(self.get_text())
                except ValueError, e:
                    raise EntryValueError, e.args

    def precision(step):
        # calculate a reasonable precision from a given step size
        if math.fabs(step) >= 1.0 or step == 0.0:
            digits = 0
        else:
            digits = abs(math.floor(math.log10(math.fabs(step))));
        if digits > 20:
            digits = 20
        return int(digits)

    class SliderEntry(gtk.HScale):
        # bounds is (upper, lower, step)
        def __init__(self, default=0, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            # modif C.L.
            def react(adj):
                return self.get_toplevel().la_previ.invalidate()
            #~ self.adj.connect('value-changed', lambda adj: self.get_toplevel().la_previ.invalidate())
            self.adj.connect('value-changed', react)
            # end modif C.L.
            gtk.HScale.__init__(self, self.adj)
            self.set_digits(precision(step))

        def get_value(self):
            return self.adj.value

    class SpinnerEntry(gtk.SpinButton):
        # bounds is (upper, lower, step)
        def __init__(self, default=10, bounds=(0, 100, 5), **kwargs): # modif C.L.
            step = bounds[2]
            self.adj = gtk.Adjustment(default, bounds[0], bounds[1],
                                      step, 10 * step, 0)
            self.adj.connect('value-changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.
            gtk.SpinButton.__init__(self, self.adj, step, precision(step))


    class ToggleEntry(gtk.ToggleButton):
        #~ def __init__(self, default=0):
        def __init__(self, default=0,labels=[_("DON'T USE"),_("YES")], **kwargs): # modif C.L.
            
            gtk.ToggleButton.__init__(self)
            self.labels = labels
            self.label = gtk.Label(labels[0])
            self.add(self.label)
            self.label.show()

            #~ self.connect("toggled", self.changed)
            # modif C.L.
            def react(tog):
                self.changed(tog)
                return kwargs['dialg'].la_previ.invalidate()
            self.connect("toggled", react)
            # end modif C.L.


            self.set_active(default)

        def changed(self, tog):
            if tog.get_active():
                self.label.set_text(self.labels[1])
            else:
                self.label.set_text(self.labels[0])

        def get_value(self):
            return self.get_active()

    class RadioEntry(gtk.VBox):
        def __init__(self, default=0, items=((_("YES"), 1), (_("No"), 0)), **kwargs): # modif C.L.
            gtk.VBox.__init__(self, homogeneous=False, spacing=2)

            button = None

            for (label, value) in items:
                button = gtk.RadioButton(button, label)
                self.pack_start(button)
                button.show()

                button.connect("toggled", self.changed, value)

                if value == default:
                    button.set_active(True)
                    self.active_value = value

        def changed(self, radio, value):
            if radio.get_active():
                self.active_value = value

        def get_value(self):
            return self.active_value

    class ComboEntry(gtk.ComboBox):
        def __init__(self, default=0, items=(), **kwargs): # modif C.L.
            store = gtk.ListStore(str)
            for item in items:
                store.append([item])

            gtk.ComboBox.__init__(self, model=store)

            cell = gtk.CellRendererText()
            self.pack_start(cell)
            self.set_attributes(cell, text=0)

            self.set_active(default)
            self.connect('changed', lambda adj: kwargs['dialg'].la_previ.invalidate()) # modif C.L.

        def get_value(self):
            return self.get_active()

    def FileSelector(default="", title=None, **kwargs): # modif C.L.
        # FIXME: should this be os.path.separator?  If not, perhaps explain why?
        if default and default.endswith("/"):
            if default == "/": default = ""
            return DirnameSelector(default)
        else:
            return FilenameSelector(default, title=title, save_mode=False)

    class FilenameSelector(gtk.HBox):
        #gimpfu.FileChooserButton
        def __init__(self, default, save_mode=True, title=None, **kwargs): # modif C.L.
            super(FilenameSelector, self).__init__()
            if not title:
                self.title = _("Python-Fu File Selection")
            else:
                self.title = title
            self.save_mode = save_mode
            box = self
            self.entry = gtk.Entry()
            image = gtk.Image()
            image.set_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
            self.button = gtk.Button()
            self.button.set_image(image)
            box.pack_start(self.entry)
            box.pack_start(self.button, expand=False)
            self.button.connect("clicked", self.pick_file)
            if default:
                self.entry.set_text(default)

        def show(self):
            super(FilenameSelector, self).show()
            self.button.show()
            self.entry.show()

        def pick_file(self, widget):
            entry = self.entry
            dialog = gtk.FileChooserDialog(
                         title=self.title,
                         action=(gtk.FILE_CHOOSER_ACTION_SAVE
                                     if self.save_mode else
                                 gtk.FILE_CHOOSER_ACTION_OPEN),
                         buttons=(gtk.STOCK_CANCEL,
                                gtk.RESPONSE_CANCEL,
                                gtk.STOCK_SAVE
                                     if self.save_mode else
                                gtk.STOCK_OPEN,
                                gtk.RESPONSE_OK)
                        )
            dialog.set_alternative_button_order ((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))
            dialog.show_all()
            response = dialog.run()
            if response == gtk.RESPONSE_OK:
                entry.set_text(dialog.get_filename())
            dialog.destroy()

        def get_value(self):
            return self.entry.get_text()


    class DirnameSelector(gtk.FileChooserButton):
        def __init__(self, default="", **kwargs): # modif C.L.
            gtk.FileChooserButton.__init__(self,
                                           _("Python-Fu Folder Selection"))
            self.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
            if default:
                self.set_filename(default)

        def get_value(self):
            return self.get_filename()

    # modif C.L.
    class ColorSelector(gimpui.ColorSelector):
        def __init__(self, default=gimpcolor.RGB(1.0, 0, 0), **kwargs):
            ret = gimpui.ColorSelector.__init__(self, default)
            self.connect('color-changed', lambda _: kwargs['dialg'].la_previ.invalidate())
            return ret
    class FontSelector(gimpui.FontSelector):
        def __init__(self, default="Sans", **kwargs):
            ret = gimpui.FontSelector.__init__(self, default)
            self.connect('font-set', lambda _1, _2, _3: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PatternSelector(gimpui.PatternSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PatternSelector.__init__(self, default)
            self.connect('pattern-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class GradientSelector(gimpui.GradientSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.GradientSelector.__init__(self, default)
            self.connect('gradient-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    class PaletteSelector(gimpui.PaletteSelector):
        def __init__(self, default="", **kwargs):
            ret = gimpui.PaletteSelector.__init__(self, default)
            self.connect('palette-set', lambda *_: kwargs['dialg'].la_previ.invalidate())
            return ret
    # end modif C.L.
    class DrawCanvas(gtk.DrawingArea):
        def __init__(self,default, size=(100,100)):
            gtk.DrawingArea.__init__(self)
            self.set_size_request(size[0],size[1])
        def get_value(self):
            return 0

    _edit_mapping = {
            PF_INT8        : IntEntry,
            PF_INT16       : IntEntry,
            PF_INT32       : IntEntry,
            PF_FLOAT       : FloatEntry,
            PF_STRING      : StringEntry,
            PF_COLOR       : ColorSelector, # modif C.L.
            PF_ITEM        : IntEntry,  # should handle differently ...
            PF_IMAGE       : gimpui.ImageSelector,
            PF_LAYER       : gimpui.LayerSelector,
            PF_CHANNEL     : gimpui.ChannelSelector,
            PF_DRAWABLE    : gimpui.DrawableSelector,
            PF_VECTORS     : gimpui.VectorsSelector,

            PF_TOGGLE      : ToggleEntry,
            PF_SLIDER      : SliderEntry,
            PF_SPINNER     : SpinnerEntry,
            PF_RADIO       : RadioEntry,
            PF_OPTION      : ComboEntry,

            PF_FONT        : FontSelector,
            PF_FILE        : FileSelector,
            PF_FILENAME    : FilenameSelector,
            PF_DIRNAME     : DirnameSelector,
            PF_BRUSH       : gimpui.BrushSelector,
            PF_PATTERN     : PatternSelector,  # modif C.L.
            PF_GRADIENT    : GradientSelector, # modif C.L.
            PF_PALETTE     : PaletteSelector,  # modif C.L.
            PF_TEXT        : TextEntry,
            PF_TAB         : TextEntry,
            PF_DRAW        : DrawCanvas,
            PF_TITLE       : LabelTitle,
    }
    # JFGARCIA - modified: to pass args to the on_run
    if on_run:
        on_run(start_params+tuple(params))

    dialog = gimpui.Dialog(proc_name, "python-fu", None, 0, None, proc_name, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OK, gtk.RESPONSE_OK))

    dialog.set_alternative_button_order((gtk.RESPONSE_OK, gtk.RESPONSE_CANCEL))

    dialog.set_transient()

    # modif C.L.
    hboxPrincipal = gtk.HBox(spacing=10)
	
    #dialog.vbox.pack_start(hboxPrincipal)			        # modif MrQ
    swin = gtk.ScrolledWindow()                             # New line MrQ
    swin.add_with_viewport(hboxPrincipal)                   # New Line MrQ
    dialog.vbox.pack_start(swin)                            # New Line MrQ
    swin.show()                                             # New Line MrQ	

    hboxPrincipal.show()
    
    dialog.la_previ = gimpui.DrawablePreview(start_params[1])
    dialog.la_previ.set_update(False)						# disable Preview
    dialog.la_previ.set_size_request(516,516)               # New Line MrQ	
    dialog.la_previ.la_img = start_params[0]
    dialog.la_previ.la_layer = start_params[1]
    hboxPrincipal.pack_start(dialog.la_previ)

    dialog.set_default_size(920, 680)  					# New Line MrQ
    dialog.la_previ.show()
    # end modif C.L.

    vbox = gtk.VBox(False, 2)
    vbox.set_border_width(2)
    #vbox.set_size_request(-10, 0)                           # modif C.L. + MrQ
    #~ dialog.vbox.pack_start(vbox)
    hboxPrincipal.pack_start(vbox) # modif C.L.
    vbox.show()

    if blurb:
        if domain:
            try:
                (domain, locale_dir) = domain
                trans = gettext.translation(domain, locale_dir, fallback=True)
            except ValueError:
                trans = gettext.translation(domain, fallback=True)
            blurb = trans.ugettext(blurb)
        box = gimpui.HintBox(blurb)
        vbox.pack_start(box, expand=False)
        box.show()

    table = gtk.Table(1, 2, False)
    table.set_row_spacings(2)
    table.set_col_spacings(2)
    vbox.pack_start(table, expand=True)
    table.show()

    notebook = gtk.Notebook()
    tables=[]
    for i in range(len(params)):
        if params[i][0]==PF_TAB:
            vals = params[i][4]
            currPage = params[i][3]

            for n in vals:
                tbl = gtk.Table(2, 2, False)
                tbl.set_row_spacings(5)
                tbl.set_col_spacings(5)
                tbl.show()
                tables.append({'table':tbl,'rows':0, 'totCols':0})
                notebook.append_page(tbl, gtk.Label(n))
                notebook.set_current_page(currPage)

    if len(tables)>0:
        table.attach(notebook, 0,6,0,1)
        notebook.show()
    else:
        tables.append({'table':table,'rows':0})
    # modif C.L.
    def ifinvalidated(p_la_previ):
        
        # get dimension
        #===============
        x, y = p_la_previ.get_position()
        w, h = p_la_previ.get_size()
        
        # Effet
        #=========
        # has alpha
        lb_alpha = pdb.gimp_drawable_has_alpha(p_la_previ.la_layer)
        
        # img, layer temporaires
        tempImg = pdb.gimp_image_new(w, h, 0)
        #~ tempLayer = pdb.gimp_layer_new(tempImg, w, h, 0, 'temp', 100, 0)
        tempLayer = pdb.gimp_layer_new(tempImg, w, h, lb_alpha, 'temp', 100, 0)
        pdb.gimp_image_insert_layer(tempImg, tempLayer, None, 0)
        
        # copie orig dans tempo
        rgnPreviIn = p_la_previ.la_layer.get_pixel_rgn(x, y, w, h)
        rgnTemp = tempLayer.get_pixel_rgn(0, 0, w, h)
        rgnTemp[0:w, 0:h] = rgnPreviIn[x:x+w, y:y+h]
        
        # effect
        tempImg.undo_group_start()
        tempImg.disable_undo()
        
        params = []

        try:
            for wid in edit_wids:
                params.append(wid.get_value())
        except EntryValueError:
            warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
        else:
            try:
                paramsTot = (tempImg, tempLayer) + tuple(params)
                _set_defaults(proc_name, start_params + tuple(params))
                function(*paramsTot)
                
                res = tempImg.active_layer
                
                if not lb_alpha: pdb.gimp_layer_flatten(res)
                
                tempImg.undo_group_end()
                
                rgnRes = res.get_pixel_rgn(0, 0, w, h)
                
                # mise dans previ
                #================
                p_la_previ.draw_buffer(rgnRes[0:w, 0:h], int(3+lb_alpha)*w)
                
                pdb.gimp_image_delete(tempImg)
                
                #pdb.gimp_display_new(tempImg)
            except CancelError:
                pass
            except Exception as e:
                dialog.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                error_dialog(dialog, proc_name)
                raise
    # end modif C.L.
    def response(dlg, id):
        if id == gtk.RESPONSE_OK:
            dlg.set_response_sensitive(gtk.RESPONSE_OK, False)
            dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, False)
            params = []
            try:
                for wid in edit_wids:
                    params.append(wid.get_value())
            except EntryValueError:
                warning_dialog(dialog, _("Invalid input for '%s'") % wid.desc)
            else:
                try:
                    dialog.res = run_script(params)
                except Exception:
                    dlg.set_response_sensitive(gtk.RESPONSE_CANCEL, True)
                    error_dialog(dialog, proc_name)
                    raise
        dialog.la_previ.set_update(False)	# disable Preview
        gtk.main_quit()
    dialog.la_previ.connect('invalidated', ifinvalidated) # modif C.L.
    dialog.connect("response", response)
    iniX=0
    row=0
    edit_wids = []

    for i in range(len(params)):
        paramsi = params[i]
        pf_type = paramsi[0]
        name, desc = (paramsi[1], paramsi[2])
        def_val = defaults[i]

        tab, incr, rows, connect, cols = (0, 1, 1, None, 1)
        if len(params[i])>5:
            xtra = params[i][5]
            incr = int(getXtra(xtra, 'newLine', 1))
            if incr==0 : cols = 1
            cols = getXtra(xtra, 'cols'   , cols)
            iniX = getXtra(xtra, 'iniCol' , iniX)
            rows = getXtra(xtra, 'rows'   , 1)
            tab  = getXtra(xtra, 'tab'    , 0)
            connect = getXtra(xtra,'connect', None)
        else:
            iniX = 0
        tbl = tables[tab]['table']
        row = tables[tab]['rows']


        if pf_type==PF_TAB:
            wid = _edit_mapping[pf_type](None)
        else:
            if pf_type != PF_TITLE:
                label = gtk.Label(desc)
                label.set_use_underline(True)
                label.set_alignment(0.0, 0.5)
                tbl.attach(label, iniX + 1, iniX + 2, row, row + rows, xoptions = gtk.FILL)
                label.show()

            if pf_type in (PF_SPINNER, PF_SLIDER, PF_RADIO, PF_OPTION, PF_DRAW):
                wid = _edit_mapping[pf_type](def_val, params[i][4], dialg=dialog) # modif C.L.
            elif pf_type==PF_TOGGLE:
                if isinstance(paramsi[4], list):
                    wid = _edit_mapping[pf_type](def_val, paramsi[4],dialg=dialog)
                else:
                    wid = _edit_mapping[pf_type](def_val)				


            elif pf_type==PF_TITLE:
                wid = _edit_mapping[pf_type](desc)
            else:
                wid = _edit_mapping[pf_type](def_val, dialg=dialog) # modif C.L.
            if pf_type!=PF_TITLE:
                label.set_mnemonic_widget(wid)
                tbl.attach(wid, iniX + 2, iniX + 2 + cols, row, row + rows, yoptions=0)
                iniX = iniX + cols + 1
            else:
                tbl.attach(wid, iniX, iniX+cols, row, row + rows, xoptions=gtk.FILL)
            # jfGarcia: strip html tags from toolTip
            tTip = re.sub('<[^<]+?>', '', desc)
            if pf_type != PF_TEXT:
                wid.set_tooltip_text(tTip)
            else:
                # Attach tip to TextView, not to ScrolledWindow
                wid.view.set_tooltip_text(tTip)

            wid.set_name(name)
            if connect:
                try:
                    wid.connect(connect['signal'],connect['function'])
                except Exception, e:
                    pass
            wid.show()
            wid.desc = desc

        edit_wids.append(wid)

        tables[tab]['rows'] += incr
        if incr==1:
           iniX=0

    progress_vbox = gtk.VBox(False, 6)
    vbox.pack_end(progress_vbox, expand=False)
    progress_vbox.show()

    progress = gimpui.ProgressBar()
    progress_vbox.pack_start(progress)
    progress.show()
	
    dialog.show()

    gtk.main()

    if hasattr(dialog, "res"):
        res = dialog.res
        dialog.destroy()
        return res
    else:
        dialog.destroy()
        raise CancelError

def _run(proc_name, params):
    run_mode = params[0]
    func = _registered_plugins_[proc_name][10]

    if run_mode == RUN_NONINTERACTIVE:
        return apply(func, params[1:])

    script_params = _registered_plugins_[proc_name][8]

    min_args = 0
    if len(params) > 1:
        for i in range(1, len(params)):
            param_type = _obj_mapping[script_params[i - 1][0]]
            if not isinstance(params[i], param_type):
                break

        min_args = i

    if len(script_params) > min_args:
        start_params = params[:min_args + 1]

        if run_mode == RUN_WITH_LAST_VALS:
            default_params = _get_defaults(proc_name)
            params = start_params + default_params[min_args:]
        else:
            params = start_params
    else:
       run_mode = RUN_NONINTERACTIVE

    if run_mode == RUN_INTERACTIVE:
        try:
            res = _interact(proc_name, params[1:])
        except CancelError:
            return
    else:
        res = apply(func, params[1:])

    gimp.displays_flush()

    return res

def main():
    """This should be called after registering the plug-in."""
    gimp.main(None, None, _query, _run)

def fail(msg):
    """Display an error message and quit"""
    gimp.message(msg)
    raise error, msg

def N_(message):
    return message
	
	
#####################################################################################################################################################
############################################# END fu_preview 0.1.1 by Claude Lion $ J.F. Garcia #####################################################
#####################################################################################################################################################
import pygtk
import gtk
from sys import argv, platform
from ctypes import *
import sys, os
import time
import datetime

############################################################################
# Service routine (for GEGL)
#---------------------------------------------------------------------------
def load_library (library_name):
	if platform == "linux" or platform == "linux2":
		library_name = library_name + '.so.0'
	elif platform == "win32":
		from ctypes.util import find_library
		library_name = find_library (library_name + "-0")
	else:
		raise BaseException ("TODO")
	return CDLL (library_name)

gimpC = load_library ('libgimp-2.0')

def gegl_graph(image, drawable, gegl_graph_string):
	class GeglBuffer(Structure):
		pass
	drawable_id = drawable.ID
	
	gegl = load_library ('libgegl-0.4')
	gegl.gegl_init (None, None)
	gimpC.gimp_drawable_get_shadow_buffer.restype = POINTER (GeglBuffer)
	gimpC.gimp_drawable_get_buffer.restype        = POINTER (GeglBuffer)

	x = c_int (); y = c_int (); w = c_int (); h = c_int ()
	non_empty,x,y,w,h = pdb.gimp_drawable_mask_intersect (drawable)
	args = [b"string", c_char_p (gegl_graph_string), c_void_p ()]
	
	if non_empty:
		source = gimpC.gimp_drawable_get_buffer (drawable_id)
		target = gimpC.gimp_drawable_get_shadow_buffer (drawable_id)
		gegl.gegl_render_op (source, target, "gegl:gegl", *args)
		gegl.gegl_buffer_flush (target)
		gimpC.gimp_drawable_merge_shadow (drawable_id, PushUndo = True)
		gimpC.gimp_drawable_update (drawable_id, x, y, w, h)
		gimpC.gimp_displays_flush ()
 
def GEGL_operation_MEDIANBLUR (image, drawable, uradius):
    
    # WARNING: do not replace - with _ in the name of the GEGL function (when not simple)
    # i.e: use median-blur, not median_blur

    radius=float(uradius)
    percentile=50.0
    alpha_percentile=50.0
    mystring="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f"
    gegl_graph_string=mystring % (radius, percentile, alpha_percentile)
    gegl_graph(image, drawable,gegl_graph_string)

def GEGL_operation_unsharp_mask(image, drawable, uradius, uamount):
    
    # WARNING: do not replace - with _ in the name of the GEGL function (when not simple)
    # i.e: use median-blur, not median_blur

    radius=float(uradius)
    amount=float(uamount)
    threshold=0.0
    mystring="gegl:unsharp-mask radius=%f amount=%f threshold=%f"
    gegl_graph_string=mystring % (radius, amount, threshold)
    gegl_graph(image, drawable,gegl_graph_string)
 
############################################################################
# Service routine (by MareroQ) for Frame colour
#---------------------------------------------------------------------------
def gmic_rgba_to_rgb(rgba_color):

	r = rgba_color[0]
	g = rgba_color[1]
	b = rgba_color[2]

	# write rgb in correct syntax
	rgb_color = "{0},{1},{2}".format(r,g,b)
	return rgb_color
	     
############################################################################
# include, then use: erro_msg or info_msg
#---------------------------------------------------------------------------
def erro_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(0) # box
    pdb.gimp_message ("ERROR: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###
def info_msg (msg):
    ###
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
    pdb.gimp_message ("INFO: "+msg)
    pdb.gimp_message_set_handler(currenth) # current
    ###

############################################################################
# Main
#---------------------------------------------------------------------------

def LineartFramed_2S_P (inImage, inLayer,
                    frame, background, size_fr, color_frame,XSh, YSh,
                    style, intensity,
                    regaincolours, applymorpho,scale_l 
                    ):
     
    oname = inLayer.name
    if "." in oname:
        split = oname.split(".", 1)
        oname = split[0]
    
    ts = time.time()
    st = datetime.datetime.fromtimestamp(ts).strftime("%Y-%m-%d %H.%M ")
    
    currenth = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2) # error console
#######################################################################################
	# MrQ add for "Preview"	  
    active=pdb.gimp_image_get_active_layer(inImage)	

    newBGo=pdb.gimp_layer_copy(active,1)
    pdb.gimp_image_insert_layer(inImage, newBGo, None, -1)
    pdb.gimp_image_raise_layer_to_top(inImage, newBGo)

#######################################################################################	
    newBGo = inImage.layers[0]
    newBGo.name = "new BG original"
    
    # create base for applying chosen 
    baseforeffect = newBGo.copy()
    inImage.add_layer (baseforeffect, 0)
    baseforeffect.name = "base for effect"
    
    # check and -if necessary- adjust mean
    (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect, HISTOGRAM_VALUE,
                                                     0.0, 1.0)
    if mean<117.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean<117.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.1, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
    elif mean>137.5:
        #gimp.message ("mean before="+str(mean))
        loop=0
        while (mean>137.5) and (loop<100):  
            pdb.gimp_drawable_levels (baseforeffect, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      0.9, 0.0, 1.0, False)
            (mean,B,C,D,E,F) = pdb.gimp_drawable_histogram (baseforeffect,
                                                 HISTOGRAM_VALUE, 0.0, 1.0)
            loop+=1
        #gimp.message ("mean after="+str(mean))
        baseforeffect.name = baseforeffect.name+" mean adj"
      
    # added GEGL Median Blur
    radius=float(2.00)  #3.00
    percentile=50.0
    alpha_percentile=50.0
    mystring="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f"
    gegl_graph_string=mystring % (radius, percentile, alpha_percentile)
    gegl_graph(inImage, baseforeffect, gegl_graph_string)
     
    # added GEGL Unsharp Mask
    radius=float(4.00)
    amount=float(2.00)
    threshold=0.0
    mystring="gegl:unsharp-mask radius=%f amount=%f threshold=%f"
    gegl_graph_string=mystring % (radius, amount, threshold)
    gegl_graph(inImage, baseforeffect, gegl_graph_string)
     
    #pdb.gimp_display_new (inImage)
    #return
     
    # Use some functionalities of G'MIC to assess structure of the image
    pdb.plug_in_gmic_qt(inImage,baseforeffect, 1,2,
        "-v - -texturize_canvas 80,4,0.6 ")
    texturized = pdb.gimp_image_get_active_layer (inImage)
    pdb.gimp_drawable_levels (texturized, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.0, 0.1, 0.5, False)   # cut DARKS and LIGHTS
    texturized.opacity=20.0
    # Name the BGlayer to use for styles
    newBG = pdb.gimp_image_merge_down (inImage, texturized, CLIP_TO_BOTTOM_LAYER)
    # cut of DARKS and LIGHTS
    pdb.gimp_drawable_levels (newBG, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                      1.0, 0.1, 0.9, False)   # cut BLACKS and WHITES
    
    #pdb.gimp_display_new (inImage)
    #return
    
    # START APPLYING USER CHOICES (Selected only those which give good results)
    
    # apply chosen GMIC Colors effect
    if style==0:    #Gradient Normalize
        # Apply gradient_norm
        #[G'MIC] fx_gradient_norm 0,0.5,0,60,0,0,50,50
        parms='0,0.5,0,60,0,0,50,50'    
    
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_gradient_norm "+str(parms)+" ")    
        
        LineartLayer.name = "Lineart Gradient Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
                                           
    #pdb.gimp_display_new (inImage)
    #return
         
    if style==1:    #Edge (by afre)
        # Apply afre_edge
        #[G'MIC] -afre_edge 0,1,1,1,1 #>>> 0,1,4,4,1 
        parms='0,1,4,4,1'
    
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -afre_edge "+str(parms))     
        
        LineartLayer.name = "Lineart afre_edge "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)
        tempE=LineartLayer.copy()
        inImage.add_layer (tempE, 0)
        pdb.gimp_drawable_equalize (tempE,False) # reqd, to adjust values too light
        tempE.opacity=50.0
        tempE.mode = LAYER_MODE_VIVID_LIGHT
        LineartLayer = pdb.gimp_image_merge_down (inImage, tempE,
                                                  CLIP_TO_BOTTOM_LAYER)
    
    if style==2:    #Edges
        # Apply fx_edges
        #[G'MIC] fx_edges 0,10,0,0,50,50
        parms='0,4.5,0,0,50,50'
    
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)
   
        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_edges "+str(parms))
        
        LineartLayer.name = "Lineart edges Layer "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_drawable_invert (LineartLayer,False)    # already inverted
        
    if style==3:    #JR_Gradient_Norm
        # Apply jr_gradient_norm
        #[G'MIC] fx_jr_gradient_norm 0,0.45,10,50,1,0,1,1,2,0,0,50,50
        parms='0,0.45,10,50,1,0,1,1,2,0,0,50,50 '
    
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)
        

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_jr_gradient_norm "+str(parms))
           
        LineartLayer.name = "Lineart JR Grad Norm "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_drawable_invert (LineartLayer,False)
        
    #pdb.gimp_display_new (inImage)
    #return
        
    if style==4:    #Thin_Edges
        # Apply thin_edges
        #[G'MIC] fx_thin_edges 0,9.5,0,0,50,50
        parms='0,9.5,0,0,50,50'
        
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)
        
        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_thin_edges "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart: Thin_Edges "+str(parms)        
        
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #already inverted
    
    # -----------------------------------------------------------------------------
    # CONVOLVE: various types
    # -----------------------------------------------------------------------------

    if style==5:    #Convolve (Laplacian)
        # Apply convolve
        # [G'MIC] fx_convolve 11,1,"0,1,0;1,-4,1;0,1,0",0,5,2,0,50,50  
        parms='11,1,"0,1,0;1,-4,1;0,1,0",0,5,2,0,50,50' #test
  
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Laplacian "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        pdb.gimp_drawable_invert (LineartLayer,False)
        
        tempE=LineartLayer.copy()
        inImage.add_layer (tempE, 0)
        pdb.gimp_drawable_equalize (tempE,False) # reqd, to adjust values too light
        tempE.opacity=90.0
        tempE.mode = LAYER_MODE_VIVID_LIGHT
        LineartLayer = pdb.gimp_image_merge_down (inImage, tempE,
                                                  CLIP_TO_BOTTOM_LAYER)
        
    # ------------------------------
    # CONVOLVE: various types double
    # ------------------------------
                        
    if style==6:    #Convolve (Prewitt X+Y)
        # Apply Prewitt X
        #[G'MIC] fx_convolve 5,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50
        parms='5,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'  
  
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Prewitt X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        
        # Apply Prewitt Y
        # [G'MIC] fx_convolve 6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50  
        parms='6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'  
  
        LineartLayer2 = newBG.copy()
        inImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Prewitt Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (inImage)
        #return
        
        parms='5&6,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (inImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Prewitt X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
        
    if style==7:    #Convolve (Sobel X+Y)
        # Apply Sobel X
        # [G'MIC] fx_convolve 7,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50  
        parms='7,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
  
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Sobel X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Sobel Y
        #[G'MIC] fx_convolve 8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50   
        parms='8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        inImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Sobel Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (inImage)
        #return
        
        parms='7&8,1,"0,1,0;1,-4,1;0,1,0",0,3,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (inImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Sobel X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (inImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==8:    #Convolve (Potinv X+Y)
        # Apply Potinv X
        # [G'MIC] fx_convolve 9,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50  
        parms='9,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
   
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Potinv X "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Potinv Y
        # [G'MIC] fx_convolve 10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50   
        parms='10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        inImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Potinv Y "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 100.0
              
        #pdb.gimp_display_new (inImage)
        #return
        
        parms='9&10,1,"0,1,0;1,-4,1;0,1,0",0,17,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (inImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Potinv X+Y "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0

        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==9:    #Convolve (Cross 1+2)
        # Apply Cross 1
        # [G'MIC] fx_convolve 12,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50 
         
        parms='12,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
   
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Convolve Cross 1 "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        # Apply Cross 2
        #[G'MIC]  fx_convolve 13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50 
        parms='13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
  
        LineartLayer2 = newBG.copy()
        inImage.add_layer (LineartLayer2, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer2, 1,0,
        "-v - -fx_convolve "+str(parms)+" ")     
        
        LineartLayer2.name = "Lineart Convolve Cross 2 "+str(parms)
         
        LineartLayer2.mode = LAYER_MODE_LIGHTEN_ONLY
        LineartLayer2.opacity = 50.0
              
        #pdb.gimp_display_new (inImage)
        #return
        
        parms='12&13,1,"0,1,0;1,-4,1;0,1,0",0,10,27,0,50,50'
        LineartLayer=pdb.gimp_image_merge_down (inImage,LineartLayer2,0)
        LineartLayer.name = "Lineart Convolve Cross 1+2 "+str(parms)
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
        #pdb.gimp_display_new (inImage)
        #return
        
        pdb.gimp_drawable_invert (LineartLayer,False)    
        
    if style==10:    #Isophotes #new
        # Apply isophotes
        # [G'MIC] fx_isophotes 31,0.75,1,0,50,50    
        parms='31,0.75,1,0,50,50'
  
        LineartLayer = newBG.copy()
        inImage.add_layer (LineartLayer, 0)

        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_isophotes "+str(parms)+" ")     
        
        LineartLayer.name = "Lineart Isophotes "+str(parms)
         
        LineartLayer.mode = LAYER_MODE_NORMAL
        LineartLayer.opacity = 100.0
        
    ##################################################################
    
    # COMMON ******************************************************
    
    # Style layer  
    
    StyleLayer = pdb.gimp_layer_new_from_visible (inImage,
                                                  inImage,
                                                  "Visible layer for Style")
    inImage.add_layer (StyleLayer, 0)
       
    # APPLY LEVELS
    pdb.gimp_drawable_levels (StyleLayer, HISTOGRAM_VALUE, 0.0, 1.0, False,
                                                5.0, 0.1, 0.9, False)
    
    StyleLayer.mode = LAYER_MODE_BURN   #MULTIPLY #NORMAL 
    StyleLayer.opacity = 90.0   #100.0
        
    #pdb.gimp_display_new (inImage)
    #return
     
    # added GEGL Median Blur
    radius=float(0.50)
    percentile=50.0
    alpha_percentile=50.0
    gegl_graph_string="gegl:median-blur radius=%f percentile=%f alpha-percentile=%f" % (radius, percentile, alpha_percentile)
    gegl_graph(inImage, StyleLayer, gegl_graph_string)
     
    # added BG desaturated on top
    BGdesat = newBGo.copy()
    inImage.add_layer (BGdesat, 0)
    pdb.gimp_drawable_desaturate (BGdesat, DESATURATE_LUMINANCE)
    if style==10:    #Isophotes #new
        BGdesat.mode = LAYER_MODE_SCREEN
    else:    
        BGdesat.mode = LAYER_MODE_HARDLIGHT #SOFTLIGHT #OVERLAY #BURN #LINEAR_BURN   
    BGdesat.name = "Original desaturated"
    BGdesat.opacity = 90.0  #100.0
    # APPLY LEVELS on Original desaturated
    pdb.gimp_drawable_levels (BGdesat, HISTOGRAM_VALUE,
                                  0.0, 1.0, False,
                                  1.0, 0.1, 0.9, False)
    
    prefinal_layer = pdb.gimp_layer_new_from_visible (inImage, inImage,
                                                      "Pre_Outcome")
    inImage.add_layer (prefinal_layer, 0)
    prefinal_layer.name = "Preliminary Outcome"
    prefinal_layer.mode = LAYER_MODE_NORMAL
        
    #pdb.gimp_display_new (inImage)
    #return
    
    if applymorpho>0:
        # bigger size, to minimize the effect
        pdb.gimp_layer_scale (prefinal_layer,
                              prefinal_layer.width*4/3,
                              prefinal_layer.height*4/3,
                              True)
        if applymorpho==1:
            # apply morpho-dilation
            # fx_morphological 1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50
            parms='1,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50'
        if applymorpho==2:
            # apply morpho-erosion
            # fx_morphological 0,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50
            parms='0,2,5,"1,0,1; 0,1,0; 1,0,1",0,0,0,0,0,50,50'
        pdb.plug_in_gmic_qt(inImage,LineartLayer, 1,0,
        "-v - -fx_morphological "+str(parms)+" ")     
        # back to original size
        pdb.gimp_layer_scale (prefinal_layer,
                              newBG.width,
                              newBG.height,
                              True)
    if regaincolours==True:
        # RE-APPLY LEVELS on PreFinal
        pdb.gimp_drawable_levels (prefinal_layer, HISTOGRAM_VALUE,
                                  0.0, 1.0, False,
                                  1.0, 0.1, 0.9, False)
        BGonTop = newBGo.copy()
        inImage.add_layer (BGonTop, 0)
        BGonTop.mode = LAYER_MODE_LCH_COLOR
        BGonTop.opacity = 100.0
    else:
        pdb.gimp_drawable_desaturate (prefinal_layer, DESATURATE_LUMINANCE)
        
    #pdb.gimp_display_new (inImage)
    #return
     
    # outcome
    
    outcome = pdb.gimp_layer_new_from_visible (inImage, inImage, "Outcome")
    inImage.add_layer (outcome, 0)
    outcome.name = "Outcome (unframed)"
        
    #pdb.gimp_display_new (inImage)
    #return
    
    # G'MIC Auto Balance outcome
    # gcd_auto_balance 30,0,0,1,0
    outcomeb = outcome.copy()
    inImage.add_layer (outcomeb, 0)
    parms='30,0,0,1,0'
    pdb.plug_in_gmic_qt(inImage,outcomeb, 1,0,
        "-v - -gcd_auto_balance "+str(parms)+" ")
    outcomeb.opacity = 30.0
    outcome=pdb.gimp_image_merge_down (inImage,outcomeb,0)
    
    # apply GMIC Frame Blur  
    # fx_frame_blur 10,10,0,5,1,0,128,128,128,0,0,255,255,255,2,2,1,0,0.5,0.5,0
    hex = gmic_rgba_to_rgb(color_frame)

    Round=["0.0","1.0","0.5","0.2","0.1"]	
    Rnd=str(Round[frame])
    #XSh = min(10,int(size_fr/6))
    #YSh = min(10,int(size_fr/6))
    #OuS = max(2,int(size_fr/2))	
    #parms_1=str(SH)+','+str(SV)+',0,5,'+str(Rnd)+',0,128,128,128,0,5'
    parms=str(background)+','+str(background)+',0,5,'+str(Rnd)+',0,128,128,128,0,' + str(size_fr) + ',' + str(hex) + ',' + str(XSh/5) +',' +str(YSh/5) + ',1,0,0.5,0.5,0 '	
    pdb.plug_in_gmic_qt(inImage, outcome, 1,2, "fx_frame_blur "+str(parms))
    Framed = pdb.gimp_image_get_active_layer(inImage)
    Framed.mode = LAYER_MODE_NORMAL
 

         
    #pdb.gimp_display_new (inImage)
     
    stylename = ["Gradient Normalize","Afre Edge",
                 "Edges","Gradient Norm JR",
                 "Thin_Edges",
                 "Convolve Prewitt X","Convolve Prewitt Y",
                 "Convolve Sobel X","Convolve Sobel Y",
                 "Convolve Potinv X","Convolve Potinv Y",
                 "Convolve Laplacian",
                 "Convolve Cross 1","Convolve Cross 2",
                 "Convolve Prewitt X+Y",
                 "Convolve Sobel X+Y",
                 "Convolve Potinv X+Y",
                 "Convolve Cross 1+2",
                 "Isophotes",
                  ]     
    Framed.name = st+oname+" with a framed line art v.1.2 effect, style "
    Framed.name = Framed.name+" "+str(stylename[style])
    if regaincolours==True:
        Framed.name = Framed.name+" (coloured)"
    if applymorpho==1:
        Framed.name = Framed.name+" (reducing black strokes)"
    elif applymorpho==2:
        Framed.name = Framed.name+" (increasing black strokes)"
             
    Framed2 = Framed.copy()
    inImage.add_layer (Framed2, 0)
    Framed2.mode = LAYER_MODE_LINEAR_BURN
    if intensity==0:
        Framed2.opacity=0.0
    elif intensity==1:
        Framed2.opacity=25.0
    elif intensity==2:
        Framed2.opacity=50.0
    elif intensity==3:
        Framed2.opacity=75.0
    else:
        Framed2.opacity = 100.0
    Framed=pdb.gimp_image_merge_down (inImage,Framed2,0)
    Framed.mode = LAYER_MODE_NORMAL
    Framed.opacity = 100.0
        
#######################################################################################
	# add for "Preview"														           
    view = pdb.gimp_layer_new_from_visible(inImage, inImage, "Preview from Visible")
    pdb.gimp_image_insert_layer(inImage, view, None, 0)

#######################################################################################		
    if scale_l==1:
        pdb.gimp_layer_scale_full(view, inLayer.width, inLayer.height,0, 3) 
        pdb.gimp_image_resize_to_layers(inImage)
    return
             
### Registration
whoiam="\n"+os.path.abspath(sys.argv[0])
author="dinasset"
menu="<Image>/Diego/SimpleFilters/"
copyrightYear="2023"
desc="Line Art Framed v.1.2S_P.\nInitial image must be set to: Image ➤ Precision\n'8 bit integer' and 'Perceptual gamma (sRGB)'"

register(
    "LineartFramed_2S_P",
    desc,desc,
    author,author,
    copyrightYear,
    "Line Art Framed v.1.2S P...",
    "*",
    [
        (PF_IMAGE, "image", "Input image", None),
        (PF_DRAWABLE, "drawable", "Input drawable", None),
        (PF_OPTION, "frame", "Choose frame format: ", 0,
                     ["square","round","half round",
                      "0.2 rounded square","0.1 rounded square"
                      ]),
		(PF_SPINNER, "background",    "Size background [ %]: ", 10, (0, 100, 1)),			    
        (PF_SPINNER, "size_fr",    "Frame thickness; ", 5, (0, 50, 1)),					  
        (PF_COLOR,   "color_frame", "Color frame: ", (200,200,200),0),
        (PF_SPINNER, "XSw",    "X Shadow: ", 1, (-10, 10, 0.1)),
        (PF_SPINNER, "YSw",    "Y Shadow: ", 1, (-10, 10, 0.1)),
		
        (PF_OPTION, "baseeffect", "Choose GMIC base effect: ", 0,
                     ["Gradient Normalize","Afre Edge",
                      "Edges","Gradient Norm JR",
                      "Thin_Edges (very slow)",
                      "Convolve Laplacian",
                      "Convolve Prewitt X+Y",
                      "Convolve Sobel X+Y",
                      "Convolve Potinv X+Y",
                      "Convolve Cross 1+2",
                      "Isophotes",					  
                      ]),
        (PF_OPTION, "flook", "Choose final look intensity: ", 2,
                     ["*base","delicate","average",
                      "intense","very intense",
                      ]),
        (PF_TOGGLE, "getcolours", "Regain colours for\nfinal outcome? ",1, ["DON'T USE","YES"]),
        (PF_OPTION, "morpho", "Apply morphological filter\nfor final outcome? ",
                                0,["*none","Dilate","Erode"]),
        (PF_TOGGLE, "scale_l", "Scale layer to\noriginal size? ", 1, ["DON'T USE","YES"]),								
        #(PF_TOGGLE, "flatten", "Flatten Image?", False,0),
    ],
    [],
    LineartFramed_2S_P,
    menu=menu
)

main()
